# -*- coding: utf-8 -*-
# pylint: disable=invalid-name, missing-docstring, line-too-long
# Generated by Django 1.10.1 on 2016-09-14 20:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django_memdb.mixins


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Environment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(blank=True, default='', max_length=64)),
                ('store', models.URLField(blank=True)),
                ('ident', models.CharField(blank=True, max_length=56, unique=True)),
                ('envar', models.TextField(blank=True)),
            ],
            bases=(models.Model, django_memdb.mixins.PeristentInMemoryDB),
        ),
        migrations.CreateModel(
            name='Inheritance',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ascendant', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ascendant', to='django_cenvars.Environment')),
                ('offspring', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='offspring', to='django_cenvars.Environment')),
            ],
            bases=(models.Model, django_memdb.mixins.PeristentInMemoryDB),
        ),
        migrations.CreateModel(
            name='Key',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=128, unique=True)),
            ],
            bases=(models.Model, django_memdb.mixins.PeristentInMemoryDB),
        ),
        migrations.CreateModel(
            name='Variable',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.TextField(blank=True, null=True)),
                ('environment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_cenvars.Environment')),
                ('key', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_cenvars.Key')),
            ],
            bases=(models.Model, django_memdb.mixins.PeristentInMemoryDB),
        ),
    ]
